package vc.thinker.mqtt.proxyclient.entity.up;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.fastjson.serializer.JSONSerializable;
import com.alibaba.fastjson.serializer.JSONSerializer;
import com.google.common.collect.Lists;
import com.google.common.collect.Maps;
import lombok.Data;
import lombok.experimental.Accessors;

import java.io.Serial;
import java.io.Serializable;
import java.lang.reflect.Type;
import java.util.ArrayList;
import java.util.Map;
import java.util.Objects;

/**
 * 原始数据
 *
 * @author XFY
 * @since 2022/10/24 18:03
 **/
@Data
@Accessors(chain = true)
public class MeterData implements Serializable, JSONSerializable {

    @Serial
    private static final long serialVersionUID = -6256921296477327221L;

    /**
     * 时间戳
     */
    private Long timestamp;
    private Map<String, Object> meterDataMap;
    private static final String TIME_PREFIX = "T_";
    private static final String DEVICE_CODE = "deviceCode";

    public MeterData() {
        this.meterDataMap = Maps.newHashMap();
    }

    /**
     * 设置设备编号
     */
    public MeterData setDeviceCode(String deviceCode) {
        meterDataMap.put(DEVICE_CODE, deviceCode);
        return this;
    }

    /**
     * 获取设备编号
     */
    public String getDeviceCode() {
        return String.valueOf(meterDataMap.get(DEVICE_CODE));
    }

    /**
     * 添加属性数据
     *
     * @param code  属性编号
     * @param value 属性值
     * @return MeterData
     */
    public MeterData putAttrData(String code, Object value) {
        Objects.requireNonNull(timestamp, "时间戳不能为空");
        putAttrData(timestamp, code, value);
        return this;
    }

    /**
     * 添加属性数据
     *
     * @param timestamp 时间戳
     * @param code      属性编号
     * @param value     属性值
     * @return MeterData
     */
    public MeterData putAttrData(Long timestamp, String code, Object value) {
        String key = TIME_PREFIX + timestamp;
        DataBucket dataBucket = (DataBucket) meterDataMap.computeIfAbsent(key, k -> new DataBucket());
        dataBucket.putAttrData(code, value);
        return this;
    }

    /**
     * 添加事件数据
     *
     * @param code  事件编号
     * @param value 事件值
     * @return MeterData
     */
    public MeterData putEventData(String code, Object value) {
        Objects.requireNonNull(timestamp, "时间戳不能为空");
        putEventData(timestamp, code, value);
        return this;
    }

    /**
     * 添加事件数据
     *
     * @param timestamp 时间戳
     * @param code      事件编号
     * @param value     事件值
     * @return MeterData
     */
    public MeterData putEventData(Long timestamp, String code, Object value) {
        String key = TIME_PREFIX + timestamp;
        DataBucket dataBucket = (DataBucket) meterDataMap.computeIfAbsent(key, k -> new DataBucket());
        dataBucket.putEventData(code, value);
        return this;
    }

    public static void main(String[] args) {
        ArrayList<MeterData> list = Lists.newArrayList();
        MeterData v2 = new MeterData();
        v2.setDeviceCode("123").setTimestamp(System.currentTimeMillis());
        v2.putAttrData("AirGapSensor_1.airGap", 3.45);
        v2.putEventData("AirGapSensor_1.boot_up", 1);
        list.add(v2);
        System.out.println(v2);
        System.out.println(JSONArray.toJSONString(list));
    }

    /**
     * 数据包封装
     * 标识一个仪表的一次上报,内容可包含0个或多个属性、事件
     *
     * @author HeTongHao
     * @since 2022/8/8 20:39
     */
    @Data
    @Accessors(chain = true)
    public static class DataBucket implements Serializable {

        @Serial
        private static final long serialVersionUID = -7756910224854163803L;

        /**
         * 属性数据键值对
         */
        private Map<String, Object> datas;

        /**
         * 事件数据键值对
         */
        private Map<String, Object> events;

        public Map<String, Object> putAttrData(String code, Object value) {
            if (datas == null) {
                datas = Maps.newHashMap();
            }
            datas.put(code, value);
            return datas;
        }

        public Map<String, Object> putEventData(String code, Object value) {
            if (events == null) {
                events = Maps.newHashMap();
            }
            events.put(code, value);
            return events;
        }
    }

    @Override
    public void write(JSONSerializer serializer, Object fieldName, Type fieldType, int features) {
        serializer.write(meterDataMap);
    }

    @Override
    public String toString() {
        return JSONObject.toJSONString(meterDataMap);
    }
}
